@@ -467,13 +467,25 @@ class SiteKey(FirebaseDoc):
     validTaskStatusCodes: List[int]
     validEventTypes: List[int]
     customizations: Dict[str, Any]
+    kpiConfig: Dict[str, Any] = attrib(factory=dict)
     unapprovedUsers: List[str] = attrib(factory=list)
 
     schema: ClassVar[Dict] = {
         "name": {"type": "string", "required": True},
+        "kpiConfig": {
+            "type": "dict",
+            "required": True,
+            "keysrules": {"type": "string"},
+        },
         "timezone": {"type": "string", "required": True, "check_with": valid_timezone},
         "managingCompanyID": {"type": "string", "required": True},
-        "unapprovedUsers": {"type": "list", "schema": {"type": "string",},},
+        "unapprovedUsers": {
+            "type": "list",
+            "required": True,
+            "schema": {
+                "type": "string",
+            },
+        },
         "validCraftTypes": {
             "type": "list",
             "required": True,
@@ -491,8 +503,8 @@ class SiteKey(FirebaseDoc):
         },
         "validEventTypes": {
             "type": "list",
-            "required": True,
-            "schema": {"type": "integer", "allowed": list(event_type_codes.values())},
+            "required": False,
+            "schema": {"type": "integer"},
         },
         "customizations": {
             "type": "dict",
@@ -505,6 +517,7 @@ class SiteKey(FirebaseDoc):
         """Create a dictionary of data ready to be written to Firestore."""
         return {
             "name": self.name,
+            "kpiConfig": self.kpiConfig,
             "timezone": self.timezone,
             "managingCompanyID": self.managingCompanyID,
             "validCraftTypes": self.validCraftTypes,
@@ -512,6 +525,7 @@ class SiteKey(FirebaseDoc):
             "validTaskStatusCodes": self.validTaskStatusCodes,
             "validEventTypes": self.validEventTypes,
             "customizations": self.customizations,
+            "unapprovedUsers": self.unapprovedUsers,
         }
 
 
@@ -522,9 +536,24 @@ class SiteKeyCompany(FirebaseDoc):
     mainPointOfContact: Optional[str]
     logoPhotoURL: Optional[str]
     members: List[str]
+    canRequestCraftTypes: Union[List[int], None] = None
+    isPlantCompany: Union[bool, None] = None
 
     schema: ClassVar[Dict] = {
         "name": {"type": "string", "maxlength": 200, "required": True},
+        "canRequestCraftTypes": {
+            "type": "list",
+            "required": False,
+            "nullable": False,
+            "schema": {
+                "type": "integer",
+                "check_with": partial(
+                    in_type_code_list,
+                    type_code_dict=craft_type_codes,
+                    type_enum=CraftTypes,
+                ),
+            },
+        },
         "craftTypes": {
             "type": "list",
             "required": True,
@@ -538,6 +567,11 @@ class SiteKeyCompany(FirebaseDoc):
                 ),
             },
         },
+        "isPlantCompany": {
+            "type": "boolean",
+            "required": False,
+            "nullable": False,
+        },
         "mainPointOfContact": {
             "type": "string",
             "required": True,
@@ -553,9 +587,23 @@ class SiteKeyCompany(FirebaseDoc):
         "members": {"type": "list", "required": True, "schema": {"type": "string"}},
     }
 
+    def to_dict(self) -> dict:
+        output = super().to_dict()
+
+        # Remove optional properties from output dictionary so the class instance can
+        # pass the validation schema. Because these are non-nullable.
+        if output["canRequestCraftTypes"] is None:
+            del output["canRequestCraftTypes"]
+
+        if output["isPlantCompany"] is None:
+            del output["isPlantCompany"]
+
+        return output
+
     def to_firestore(self) -> dict:
         """Return a dictionary ready to be written to Firestore"""
-        return {
+
+        output = {
             "name": self.name,
             "craftTypes": self.craftTypes,
             "mainPointOfContact": self.mainPointOfContact,
@@ -563,6 +611,17 @@ class SiteKeyCompany(FirebaseDoc):
             "members": self.members,
         }
 
+        # Conditional added to avoid writing null to database for optional property.
+        if self.canRequestCraftTypes is None:
+            pass
+        elif len(self.canRequestCraftTypes) > 0:
+            output["canRequestCraftTypes"] = self.canRequestCraftTypes
+
+        if self.isPlantCompany is not None:
+            output["isPlantCompany"] = self.isPlantCompany
+
+        return output
+
 
 @attrs(auto_attribs=True)
 class SiteKeyUser(FirebaseDoc):
