@@ -3,7 +3,6 @@ from enum import Enum
 from pprint import pprint, pformat
 from typing import List, Union, Optional, Tuple
 
-import attr
 import cutie
 import typer
 from cerberus import Validator
@@ -241,15 +240,18 @@ def create_site_key(
         )
     if len(valid_task_types) == 0:
         valid_task_types = prompt_select_multiple_from_type_enum_with_all(
-            title="Select Task Types:", enum=TaskTypes,
+            title="Select Task Types:",
+            enum=TaskTypes,
         )
     if len(valid_task_status_codes) == 0:
         valid_task_status_codes = prompt_select_multiple_from_type_enum_with_all(
-            title="Select Task Status Codes:", enum=TaskStatus,
+            title="Select Task Status Codes:",
+            enum=TaskStatus,
         )
     if len(valid_event_types) == 0:
         valid_event_types = prompt_select_multiple_from_type_enum_with_all(
-            title="Select Event Types:", enum=EventTypes,
+            title="Select Event Types:",
+            enum=EventTypes,
         )
 
     # Convert enum lists to integer codes
@@ -289,10 +291,14 @@ def create_site_key(
         )
         raise typer.Abort()
 
+    # validate output to be written to firestore.
+    site_key_updates = site_key.to_firestore()
+    v = Validator(SiteKey.schema)
+    v.validate(site_key_updates)
+    if v.errors != {}:
+        raise ValueError(v.errors)
     # confirm and set/create new site key
-    confirm_and_set_sitekey(
-        yes=yes, site_key=site_key_id, updates=site_key.to_firestore()
-    )
+    confirm_and_set_sitekey(yes=yes, site_key=site_key_id, updates=site_key_updates)
 
     if yes:
         create_new_company(
@@ -305,6 +311,8 @@ def create_site_key(
             main_point_of_contact=company_point_of_contact,
             logo_photo_url=company_logo_photo_url,
             members=company_members,
+            is_plant_company=True,
+            can_request_craft_types=None,
         )
     else:
         confirm_text = typer.style(
@@ -322,6 +330,8 @@ def create_site_key(
                 main_point_of_contact=company_point_of_contact,
                 logo_photo_url=company_logo_photo_url,
                 members=company_members,
+                is_plant_company=True,
+                can_request_craft_types=None,
             )
     typer.secho("Finished.", fg="green")
 
@@ -337,13 +347,19 @@ def create_new_company(
     site_key: str = typer.Option(..., prompt=True),
     company_id: str = typer.Option(..., prompt=True),
     name: str = typer.Option(..., prompt=True),
+    is_plant_company: bool = typer.Option(..., prompt=True),
     craft_types: List[CraftTypes] = typer.Option(None, case_sensitive=False),
     main_point_of_contact: Optional[str] = typer.Option("", prompt=True),
     logo_photo_url: Optional[str] = typer.Option("", prompt=True),
     members: List[str] = typer.Option(None),
+    add_can_request_craft_types: bool = typer.Option(..., prompt=True),
+    can_request_craft_types: Union[None, List[CraftTypes]] = None,
 ) -> SiteKeyCompany:
+    choose_project(skip=skip_project)
 
     # Handle prompting for values if function not invoked from the CLI
+    # todo: these conditionals may never be True if Typer never passes None and always
+    #  prompts for missing arguments
     if site_key is None:
         site_key = typer.prompt("Enter the site key", type=str)
     if company_id is None:
@@ -364,6 +380,15 @@ def create_new_company(
             enum=CraftTypes,
         )
 
+    if add_can_request_craft_types is True:
+        can_request_craft_types = prompt_select_multiple_from_type_enum_with_all(
+            title="What additional Craft Types can this company request from other companies?",
+            enum=CraftTypes,
+            allow_none=True,
+        )
+        if len(can_request_craft_types) == 0:
+            can_request_craft_types = None
+
     # Make sure members is a list
     if members is None:
         members = list()
@@ -379,9 +404,13 @@ def create_new_company(
 
     # Convert to integer codes.
     craft_types = [e.code for e in craft_types]
+    if can_request_craft_types is not None:
+        can_request_craft_types = [e.code for e in can_request_craft_types]
     company = SiteKeyCompany(
         name=name,
+        canRequestCraftTypes=can_request_craft_types,
         craftTypes=craft_types,
+        isPlantCompany=is_plant_company,
         mainPointOfContact=main_point_of_contact,
         logoPhotoURL=logo_photo_url,
         members=members,
@@ -390,14 +419,17 @@ def create_new_company(
     # Validate
     spinner = Halo(text_color="blue")
     spinner.start(f"Validating data...")
-    v = Validator(SiteKeyCompany.schema, require_all=True)
-    if v.validate(attr.asdict(company)) is False:
+    updates = company.to_firestore()
+    v = Validator(company.schema)
+    v.validate(updates)
+    if v.errors != {}:
         spinner.fail()
         pprint(v.errors)
-        typer.Abort()
+        raise typer.Abort()
     spinner.succeed()
 
-    choose_project(skip=skip_project)
+    if db.does_site_key_exist(site_key=site_key) is False:
+        raise db.NotFound("Site key does not exist.")
 
     if db.does_doc_exist(f"siteKeys/{site_key}/siteKeyCompanies/{company_id}"):
         typer.secho(
@@ -406,7 +438,7 @@ def create_new_company(
         raise typer.Abort()
 
     confirm_and_set_site_key_company(
-        yes, site_key=site_key, company_id=company_id, updates=company.to_firestore()
+        yes, site_key=site_key, company_id=company_id, updates=updates
     )
 
     return company
@@ -569,7 +601,7 @@ def add_craft_detail(
 
     except (ValueError, db.NotFound) as err:
         typer.secho(f"{err}", fg="red")
-        typer.Abort()
+        raise typer.Abort()
 
 
 def parse_add_craft_detail(
@@ -786,7 +818,9 @@ def prompt_select_multiple_craft_types() -> List[CraftTypes]:
 
 
 def prompt_select_multiple_from_type_enum_with_all(
-    title: str, enum: Enum
+    title: str,
+    enum: Enum,
+    allow_none: bool = False,
 ) -> List[Enum]:
     """
     Prompt the user to select from a list of available types in an enum. Return a list
@@ -794,14 +828,24 @@ def prompt_select_multiple_from_type_enum_with_all(
     """
     enum_list = list(enum)
     enum_strings = [e.value for e in enum_list]
+    index_adjustment = 1
 
     # insert ALL options
     enum_strings.insert(0, "ALL")
+
+    # insert NONE option.
+    if allow_none is True:
+        enum_strings.insert(1, "NONE")
+        index_adjustment = 2
+
     typer.secho(title, fg=typer.colors.CYAN)
     selected_indexes = cutie.select_multiple(enum_strings, hide_confirm=True)
     if 0 in selected_indexes:
         return enum_list
-    selected_enums = [enum_list[index - 1] for index in selected_indexes]
+    elif allow_none is True and 1 in selected_indexes:
+        return list()
+
+    selected_enums = [enum_list[index - index_adjustment] for index in selected_indexes]
     return selected_enums
 
 
